//  
//  Copyright (C) 2011 Adrien Plazas
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using Gtk;
using Gdk;

namespace Strip
{
	class ComicLayout : Layout
	{
		public signal void left_page_pressed();
		public signal void right_page_pressed();
		
		string view_mode;
		PixbufRotation rotation = PixbufRotation.NONE;
		
		Pixbuf first_pixbuf;
		Pixbuf second_pixbuf;
		Image first_page = new Image();
		Image second_page = new Image();
		
		int last_width = 0;
		int last_height = 0;
		
		public ComicLayout ()
		{
			events |= EventMask.BUTTON_PRESS_MASK | EventMask.BUTTON_RELEASE_MASK;
			reset();
			set_single_mode();
			set_can_focus(true);
			// Color should be (4095, 4095, 4095)
			modify_bg(StateType.NORMAL, Color());
			button_press_event.connect(on_button_press_event);
			check_resize.connect (on_resize_event); // Ne marche pas, à arranger
			
			put(first_page, 0, 0);
			put(second_page, 0, 0);
		}
		
		/*
		 * 
		 * Le resize event se fait depuis l'extérieur !
		 * 
		 */
		public void on_resize_event()
		{
			if (last_width != get_width() || last_height != get_height() )
			{
				last_width = get_width();
				last_height = get_height();
				refresh();
			}
		}
		
		/*
		 * 
		 * À retravailler
		 * 
		 */
		private bool on_button_press_event(EventButton event)
		{
			if (event.button == 1)
			{
				if (rotation == PixbufRotation.COUNTERCLOCKWISE)
				{
					if (event.y < get_height() / 2)
					{ right_page_pressed(); }
					else
					{ left_page_pressed(); }
				}
				else if (rotation == PixbufRotation.CLOCKWISE)
				{
					if (event.y < get_height() / 2)
					{ left_page_pressed(); }
					else
					{ right_page_pressed(); }
				}
				else if (rotation == PixbufRotation.UPSIDEDOWN)
				{
					if (event.x < get_width() / 2)
					{ right_page_pressed(); }
					else
					{ left_page_pressed(); }
				}
				else
				{
					if (event.x < get_width() / 2)
					{ left_page_pressed(); }
					else
					{ right_page_pressed(); }
				}
			}

//~         if event.button == 2:
//~             self.on_fullscreen_activate(widget)
//~         if event.button == 3 and self.preferences.is_fullscreen:
//~             self.menu.popup(None, None, None, 1, gtk.get_current_event_time())
//~             self.menu.select_first(True)
			return false;
		}
		
		//Use these instead of get_allocated_*() directly
		public int get_width()
		{ return get_allocated_width(); }
		
		public int get_height()
		{ return get_allocated_height(); }
		
		private void clear()
		{
			first_page.hide();
			second_page.hide();
//~ 			remove(first_page);
//~ 			remove(second_page);
//~ 			first_page = new Image();
//~ 			second_page = new Image();
		}
		
		private void reset()
		{
			clear();
			first_pixbuf = new Pixbuf(Colorspace.RGB, true, 8, 1, 1);
			second_pixbuf = new Pixbuf(Colorspace.RGB, true, 8, 1, 1);
		}
		
		public void set_double (Pixbuf first_pb, Pixbuf second_pb)
		{
			reset();
			
			first_pixbuf = first_pb;
			second_pixbuf = second_pb;
			set_double_mode();
			refresh();
		}
		
		private int[] get_double_position(int[] size, bool is_left_page)
		{
			bool rotate = (rotation == PixbufRotation.CLOCKWISE || rotation == PixbufRotation.COUNTERCLOCKWISE);
			int offset = size[(int) rotate] * (int) is_left_page;
			if (!rotate)
			{
				return {(get_width() / 2) - offset, (get_height() - size[1]) / 2};
			}
			else
			{
				if (rotation == PixbufRotation.COUNTERCLOCKWISE)
				{ offset = size[(int) rotate] * (int) !is_left_page; }
				return {(get_width() - size[0]) / 2, get_height() / 2 - offset};
			}
		}
		
		public void set_single (Pixbuf first_pb)
		{
			reset();
			first_pixbuf = first_pb;
			set_single_mode();
			refresh();
		}
		
		public void set_width (Pixbuf first_pb)
		{
			reset();
			first_pixbuf = first_pb;
			set_width_mode();
			refresh();
		}
		
		// Mode handeling
		private void set_double_mode()
		{ view_mode = "double"; }
		
		private void set_single_mode()
		{ view_mode = "single"; }
		
		private void set_width_mode()
		{ view_mode = "width"; }
		
		public void set_rotation(PixbufRotation pixbufrotation)
		{ rotation = pixbufrotation; }
				
		private void refresh()
		{
			clear();
			if (view_mode == "double")
			{ 
				bool rotate = (rotation == PixbufRotation.CLOCKWISE || rotation == PixbufRotation.COUNTERCLOCKWISE);
				var resized_first_pb = set_pixbuf_from_pixbuf(first_pixbuf, get_width() / (2 - (int) rotate), get_height() / (2 - (int) !rotate), rotation, false);
				var resized_second_pb = set_pixbuf_from_pixbuf(second_pixbuf, get_width() / (2 - (int) rotate), get_height() / (2 - (int) !rotate), rotation, false);
				first_page.set_from_pixbuf(resized_first_pb);
				second_page.set_from_pixbuf(resized_second_pb);
				var first_position = get_double_position({resized_first_pb.width, resized_first_pb.height}, true);
				var second_position = get_double_position({resized_second_pb.width, resized_second_pb.height}, false);
				move(first_page, first_position[0], first_position[1]);
				move(second_page, second_position[0], second_position[1]);
				first_page.show();
				second_page.show();
			}
			else
			{
				var resized_first_pb = set_pixbuf_from_pixbuf(first_pixbuf, get_width(), get_height(), rotation, false);
				first_page.set_from_pixbuf(resized_first_pb);
				move(first_page, (get_width() - resized_first_pb.width) / 2, (get_height() - resized_first_pb.height) / 2);
				first_page.show();
			}
		}
	}
}
