//  
//  Copyright (C) 2011 Adrien Plazas
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using Gtk;
using Gdk;

namespace Strip
{
	public static int[] get_optimal_size(int box_width, int box_height, int image_width, int image_height)
	{
		float box_ratio = (float) box_width / (float) box_height;
		float image_ratio = (float) image_width / (float) image_height;
		if (box_ratio < image_ratio) // If the space reserved to the page is taller than the page
		{
			return {box_width, (int) (((float) box_width) * (1 / image_ratio))};
		}
		else // If the space reserved to the page is wider than the page (or have the same proportions)
		{
			return {(int) (((float) box_height) * image_ratio), box_height};
		}
	}
	
	public static Pixbuf resize_and_rotate(Pixbuf pixbuf, int width, int height, PixbufRotation rotation, bool flip)
	{
		var pix = pixbuf;
		if (flip) { pix = pixbuf.flip(false); }
		return pix.rotate_simple(rotation).scale_simple(width, height, InterpType.BILINEAR); // InterpType.INTERP_HYPER
	}
	
	public static Pixbuf set_pixbuf_from_pixbuf(Pixbuf pixbuf, int box_width, int box_height, PixbufRotation rotation, bool flip)
	{
		int[] size = {pixbuf.width, pixbuf.height};
		if (rotation == PixbufRotation.CLOCKWISE || rotation == PixbufRotation.COUNTERCLOCKWISE)
		{ size = {pixbuf.height, pixbuf.width}; }
		size = get_optimal_size(box_width, box_height, size[0], size[1]);
		Pixbuf pix = resize_and_rotate(pixbuf, size[0], size[1], rotation, flip);
		return pix;
	}
	
	public static Image set_image_from_pixbuf(Pixbuf pixbuf, int box_width, int box_height, PixbufRotation rotation, bool flip)
	{
		var im = new Image.from_pixbuf(set_pixbuf_from_pixbuf(pixbuf, box_width, box_height, rotation, flip));
		return im;
	}
}
